%% fc_lib_file_add_header_to_file
%%%%%%%%%%%%%
% help fc_lib_file_add_header_to_file
% Belongs to private library
%%%%%%%%%%%%%
% Author: M.Eng. Fernando Henrique G. Zucatelli (UFABC)
% COPYRIGHT: Free for non evil use. (Não use este código para o mal)
%%%%%%%%%%%%%
% Function to insert a string 'my_header' into the top of another file
%   The predefined header is my header with the given name of the file and
%   the present date as the date of its creation.
%%%%%%%%%%%%%
% This code is actually better than this one:
% Source: https://in.mathworks.com/matlabcentral/answers/98758-how-can-i-prepend-text-to-a-file-using-matlab
%%%%%%%%%%%%%
% version 01: 24.11.2019 -- Creation
%%%%%%%%%%%%% Example 01
% file_name = 'Teste_file_01'; str_ext = '.m';
% fc_lib_file_add_header_to_file(file_name, str_ext);
%%%%%%%%%%%%% Example 02
% file_name = 'Teste_file_02'; str_ext = '.txt';
% fc_lib_file_add_header_to_file(file_name, str_ext);
%%%%%%%%%%%%% Example 03
% file_name = 'Teste_file_03'; str_ext = '.m';
% my_header = char('header line 1','h line 2 -- extra','h line 3');
% fc_lib_file_add_header_to_file(file_name, str_ext, my_header);
%%%%%%%%%%%%%
%% algorithm
function fc_lib_file_add_header_to_file(file_name, str_ext, varargin)
%% varargin analysis
if nargin == 2
    header = fc_header(file_name);
else
    header = char(varargin{1});
end
%% main
file_name_ext = [file_name,str_ext];
tmp_file_ext = sprintf('tmp_file.%s', str_ext);
copyfile(file_name_ext, tmp_file_ext);
fid = fopen(file_name_ext,'w');
%% insert header at the top of the original file
% based on: save_file_tex from this same library
[l, c] = size(header);
for i = 1:l
    buffer = header(i,:);
    for j = c:-1:1
        if  buffer(j) == ' '
            buffer(j) = '';
        else
            break;
        end
    end
    fprintf(fid, '%s\n', buffer);
end
%% insert the copied tmp_file_ext into the original file
fid_tmp = fopen(tmp_file_ext,'r');
while 1
    % === pega uma linha do arquivo - get a file line
    tline = fgetl(fid_tmp);
    % === criterio de saida quando nao encontra mais linha - exit criterium and do not find a line
    if ~ischar(tline)
        break;
    end
    fprintf(fid, '%s\n', tline);
end
%% close files and delete tmp_file_ext
fclose(fid);
fclose(fid_tmp);
delete(tmp_file_ext);
end
%% function to alocate my predefined header
function str = fc_header(file_name)
% insert today as date of creation
dh = clock; ano = dh(1); mes = dh(2); dia = dh(3);
s = sprintf('%g.%g.%g',dia,mes,ano);
% predefined header
str = char(...
    sprintf('%%%% %s', file_name),...
    '%%%%%%%%%%%%%',...
    sprintf('%% help %s', file_name),...
    '% Belongs to private library',...
    '%%%%%%%%%%%%%',...
    '% Author: M.Eng. Fernando Henrique G. Zucatelli (UFABC)',...
    '% COPYRIGHT: Free for non evil use. (Não use este código para o mal)',...
    '%%%%%%%%%%%%%',...
    '% Description of the function',...
    '%%%%%%%%%%%%%',...
    '% Source: Link the source material when it is avaliable',...
    '%%%%%%%%%%%%%',...
    sprintf('%% version 01: %s -- Creation', s),...
    '%   Some details of this version',...
    '%   The numbers are in the "%02d" format and separate with colon ":"',...
    '%   Each version must have the date in the format dd.mm.yyyy',...
    '%%%%%%%%%%%%% Example 01',...
    '% Examples must call the function and may call other related functions',...
    '%%%%%%%%%%%%%',...
    '%% algorithm');
end
