function [T,nOfElements] = localRefinement(T,iElem,elementType)

% this routine splits a spherical element into four smaller ones

if elementType == 1 % spherical triangles
    v1 = T(1,:,iElem); v2 = T(2,:,iElem); v3 = T(3,:,iElem); 
    m1 = (v2+v3)/norm(v2+v3,2);
    m2 = (v1+v3)/norm(v1+v3,2);
    m3 = (v2+v1)/norm(v2+v1,2);
    T(:,:,iElem) = [m1;m2;m3]; T(:,:,end+1) = [v1;m3;m2];
    T(:,:,end+1) = [v2;m1;m3]; T(:,:,end+1) = [v3;m2;m1];
    nOfElements = size(T,3);
    
elseif elementType == 2 % angular elements
    theta1=T(iElem,1); theta2=T(iElem,2); phi1=T(iElem,3); phi2=T(iElem,4);
    phiC = (phi1+phi2)/2; thetaC = (theta1 + theta2)/2;
    T(iElem,:) = [thetaC theta2 phi1 phiC];
    T(end+1:end+3,:) = ...
        [thetaC theta2 phiC phi2;
        theta1 thetaC phiC phi2;
        theta1 thetaC phi1 phiC];
    nOfElements = size(T,1);
else 
    error('Element type is not implemented! Use elementType = 1 or 2')
end
end



