function [I] = sphericalElementIntegration(theta1,theta2,phi1,phi2,order)

% this routine integrates the fibre distribution in a spherical element

% get Gauss data (weights, points, Jacobian determinant)
[weights,points] = quadrature(order);
Xe=[cos(theta2) phi1; cos(theta2) phi2; cos(theta1) phi2; cos(theta1) phi1];
xGauss = XGauss(Xe,points); J = (cos(theta1)-cos(theta2))*(phi2-phi1)/4;

I = 0; % initialise integral
for iGauss = 1:size(xGauss,1)
    z = xGauss(iGauss,1); phi = xGauss(iGauss,2);
    theta = acos(z); x = sin(theta)*cos(phi); y = sin(theta)*sin(phi);
    n = [x y z];
    I = I + J*weights(iGauss)*evaluateRho(n);
end
end

function [XGauss] = XGauss(Xe,points)

nGauss = size(points,1); XGauss = zeros(nGauss,2);
xi = points(:,1); eta = points(:,2);

N1 = .25*(1-xi).*(1-eta); N2 = .25*(1+xi).*(1-eta);
N3 = .25*(1+xi).*(1+eta); N4 = .25*(1-xi).*(1+eta);

for iGauss=1:nGauss
    XGauss(iGauss,1) = N1(iGauss)*Xe(1,1) + N2(iGauss)*Xe(2,1) + N3(iGauss)*Xe(3,1) + N4(iGauss)*Xe(4,1);
    XGauss(iGauss,2) = N1(iGauss)*Xe(1,2) + N2(iGauss)*Xe(2,2) + N3(iGauss)*Xe(3,2) + N4(iGauss)*Xe(4,2);
end
end

