function [I] = sphericalTriangleIntegration(T,iElem,order)

% this routine integrates the fibre distribution in a spherical triangle

v1=T(1,:,iElem); v2=T(2,:,iElem); v3=T(3,:,iElem); % vertex coordinates
I = 0; % initialise integral

if order == 0 % vertex rule on subgrid
    I = sphericalTriangleIntegration_subgrid(v1,v2,v3,32);
    return
    
elseif order == 1 % centroid rule (not isoparametric!)
    F = (v1+v2+v3)/3;
    n = F/norm(F,2);
    a1 = acos(dot(v2,v3)); a2 = acos(dot(v1,v3)); a3 = acos(dot(v2,v1));
    s = .5*(a1+a2+a3);
    Area = 4*atan(sqrt(tan(.5*s)*tan(.5*(s-a1))*tan(.5*(s-a2))*tan(.5*(s-a3))));
    I = Area*evaluateRho(n);
    return
    
elseif order == 2 % isoparametric symmetric 3-point rule
    xi = [1/6 2/3 1/6]; eta = [1/6 1/6 2/3]; w = [1 1 1]/3;
    %xi = [0 1 0]; eta = [0 0 1]; w = [1 1 1]/3; % vertex rule
    
elseif order == 3 % isoparametric symmetric 4-point rule
    xi = [1/3 1/5 1/5 3/5]; eta = [1/3 1/5 3/5 1/5]; w = [-27 25 25 25]/48;
    
elseif order == 4 % isoparametric symmetric 6-point rule
    Data = [...
        9.1576213509770743e-2  9.1576213509770743e-2  5.4975871827660935e-2
        9.1576213509770743e-2  8.1684757298045851e-1  5.4975871827660935e-2
        8.1684757298045851e-1  9.1576213509770743e-2  5.4975871827660935e-2
        4.4594849091596489e-1  4.4594849091596489e-1  1.1169079483900574e-1
        4.4594849091596489e-1  1.0810301816807022e-1  1.1169079483900574e-1
        1.0810301816807022e-1  4.4594849091596489e-1  1.1169079483900574e-1];
    xi = Data(:,1); eta = Data(:,2); w = 2*Data(:,3);
    
else error('Quadrature order is not implemented!')
    
end

% isoparameteric integration
for i=1:length(w)
    
    % map spherical triangle into a flat one to use standard Gaussian rules
    F = v1 + eta(i)*(v2-v1) + xi(i)*(v3-v1);
    n = F/norm(F,2);
    f = norm(F,2);
    
    dGdXi = (v3-v1)/f - dot(F,v3-v1)*F/f^3;
    dGdEta = (v2-v1)/f - dot(F,v2-v1)*F/f^3;
    sigma = norm(cross(dGdXi,dGdEta),2);
    
    I = I + .5*sigma*w(i)*evaluateRho(n);
end
end



