function [I] = sphericalTriangleIntegration_subgrid(v1,v2,v3,factor)

% Burkardt algorithm:
% this routine estimates the integral of a function in a spherical triangle
% by uniformly refining it log2(factor) times and applying the vertex rule
% to each element of the subgrid
% (see people.sc.fsu.edu/~jburkardt/f_src/sphere_triangle_quad/sphere_triangle_quad.html)

level = ceil(log2(factor));
I = 0; c = 0; xi = [0 1 0]; eta = [0 0 1]; w = [1 1 1]/3;
T = zeros(3,3,1); T(1,:,1) = v1; T(2,:,1) = v2; T(3,:,1) = v3;
nOfElements = 1;

while c < level
    c = c + 1;
    
    for iElem = 1:nOfElements
        [T,~] = localRefinement(T,iElem,1);
    end
    
    nOfElements = size(T,3);
end

for iElem = 1:nOfElements
    v1 = T(1,:,iElem); v2 = T(2,:,iElem); v3 = T(3,:,iElem);
    
    for i=1:length(w)
        F = v1 + eta(i)*(v2-v1) + xi(i)*(v3-v1);
        n = F/norm(F,2);
        f = norm(F,2);
        
        dGdXi = (v3-v1)/f - dot(F,v3-v1)*F/f^3;
        dGdEta = (v2-v1)/f - dot(F,v2-v1)*F/f^3;
        sigma = norm(cross(dGdXi,dGdEta),2);
        
        I = I + .5*sigma*w(i)*evaluateRho(n);
    end
    
end
end



