import sys
sys.path.insert(0, '/home/mwollner/Schreibtisch/Additional Material')
from scipy.optimize import least_squares, fsolve

import matplotlib.pyplot as plt
import numpy as np
import fitting as fit
import constitutive as con
import bodies as bod

plt.close('all')
fit.latex_setup()
_, (ax1, ax2) = plt.subplots(1, 2, figsize=tuple(1*14*0.39370*np.array([1, 0.4])))

def residuals(parameters):
    return uniaxial(parameters) - sigma_exp

#==============================================================================

filename = '/home/mwollner/Schreibtisch/Additional Material/Experimental Data/Adventitia/Preprocessed Data/Dissected Aorta - Adventitia - Axial.csv'
csv_data = fit.read_csv(filename, 1)

adv_col1 = "/home/mwollner/Schreibtisch/Additional Material/Parameters/DFD/o_cf_adventitia_a=3,08,b=33,20,-50,72.csv"
adv_col2 = "/home/mwollner/Schreibtisch/Additional Material/Parameters/DFD/o_cf_adventitia_a=3,08,b=33,20,+50,72.csv"
def uniaxial(parameters):
    lmbd_ax = lmbd
    
    mu_g, k1 = parameters
    materials_adventitia = (con.ground_matrix('gr', mu_g),
                            con.collagen('col1', (k1, 1.3807), adv_col1),
                            con.collagen('col2', (k1, 1.3807), adv_col2))
    adventitia = bod.layer('adventitia', materials_adventitia)
    
    def traction_free_surfaces(x, y):
        lmbd_phi, = x
        F = fit.deformation_gradient(lmbd_phi, y, 0.)
        sigma_iso = adventitia.isochoric_cauchy_stress(F)
        root = sigma_iso[0,0,0] - sigma_iso[0,1,1]
        return root
        
    lmbd_phi = np.array([fsolve(traction_free_surfaces, 1., args=(y))[0] for y in lmbd_ax])  
    
    F = fit.deformation_gradient(lmbd_phi, lmbd_ax, np.zeros_like(lmbd_ax))

    sigma_iso = adventitia.isochoric_cauchy_stress(F)
        
    sigma_zz = sigma_iso[:,2,2] - sigma_iso[:,1,1]
    return sigma_zz   

lmbd_end = 1.3
lmbd, sigma_exp = fit.data_preprocessing(csv_data, 1.0, lmbd_end, skip=0, step=1)

fit.tic()
mu_g = 6.18
k1 = 27.86
# mu_g, k1 = least_squares(residuals, (5.0, 30.0), bounds=((0.0, 0.0), (np.inf, np.inf))).x
fit.toc()

print('Ground matrix stiffness \'mu_g\': %.4f' % mu_g)
print('Collagen stiffness \'k1\': %.4f' % k1)

# COD = fit.coefficient_of_determination(residuals((mu_g, k1)), sigma_exp)

# lmbd, sigma_exp = fit.data_preprocessing(csv_data, 1.0, np.inf, step=1)
# sigma_theo = uniaxial((mu_g, k1))
# data = [['Axial Stretch / 1', 'Axial Experimental Cauchy Stress / kPa', 'Axial Theoretical Cauchy Stress / kPa']] + \
#         [[lmbd[k], sigma_exp[k], sigma_theo[k]] for k in range(len(lmbd))]
# fit.write_csv(data, '/home/mwollner/Schreibtisch/Additional Material/Experimental Data/Uniaxial Passive/Adventitia/data.csv')

# data = [['Ground Matrix / kPa', 'Collagen Fibre Stiffness / kPa', 'CoD / 1', 'Maximum Fitting Stretch / 1']] + \
#         [[mu_g, k1, COD, lmbd_end]]
# fit.write_csv(data, '/home/mwollner/Schreibtisch/Additional Material/Experimental Data/Uniaxial Passive/Adventitia/parameters.csv')

lmbd, sigma_exp = fit.data_preprocessing(csv_data, 1.0, np.inf, step=100)

ax2.plot(lmbd[::2], sigma_exp[::2], color='k', marker='+', linestyle='None', label='Experiment')
lmbd = np.linspace(1.0, 1.597, 500)
ax2.plot(lmbd, uniaxial((mu_g, k1)), color='k', label='Model', lw=1.0)

ax2.set_xlabel(r'Axial Stretch $\lambda_z$ / 1')
ax2.set_xlim(1.0, 1.64)
ax2.set_xticks(np.arange(1.0, 1.65, 0.16))


ax2.set_ylabel(r'\textsc{Cauchy} Stress $\sigma^\mathrm{adv}_{zz}$ / kPa')
ax2.set_ylim(0.0,1000.0)
ax2.set_yticks(np.arange(0.0, 1001.0, 250.0))

# ax2.legend(loc='upper left', fancybox=False, edgecolor='k')

fit.annotations(r'(b)', ax2, v=1.15)



#==============================================================================
filename = '/home/mwollner/Schreibtisch/Additional Material/Experimental Data/Media/Preprocessed Data/Dissected Aorta - Media - Circumferential.csv'
csv_data = fit.read_csv(filename, 1)

med_col1 = "/home/mwollner/Schreibtisch/Additional Material//Parameters/DFD/o_cf_media_a=3,62,b=34,30,-25,14.csv"
med_col2 = "/home/mwollner/Schreibtisch/Additional Material/Parameters/DFD/o_cf_media_a=3,62,b=34,30,+25,14.csv"
med_el = "/home/mwollner/Schreibtisch/Additional Material/Parameters/DFD/o_ef_b=34,30.csv"


def uniaxial(parameters):
    lmbd_phi = lmbd
    
    c1, k1 = parameters
      
    materials_media = (con.ground_matrix('gr', mu_g),
                        con.collagen('col1', (k1, 1.3807), med_col1),
                        con.collagen('col2', (k1, 1.3807), med_col2),
                        con.elastin('el', (c1, 2.9841), med_el),
                        )
    media = bod.layer('media', materials_media)
    
    def traction_free_surfaces(x, y):
            lmbd_ax, = x
            F = fit.deformation_gradient(y, lmbd_ax, 0.)
            sigma_iso = media.isochoric_cauchy_stress(F)
            root = sigma_iso[0,0,0] - sigma_iso[0,2,2]
            return root
        
    lmbd_ax = np.array([fsolve(traction_free_surfaces, 1., args=(y))[0] for y in lmbd_phi])  
    
    F = fit.deformation_gradient(lmbd_phi, lmbd_ax, np.zeros_like(lmbd_phi))
    
    sigma_iso = media.isochoric_cauchy_stress(F)
    
    sigma_phiphi = sigma_iso[:,1,1] - sigma_iso[:,2,2]
    return sigma_phiphi

lmbd_end = 1.45
lmbd, sigma_exp = fit.data_preprocessing(csv_data, 1.0, lmbd_end, skip=350, step=1)
fit.tic()
c1 = 62.97
k1 = 10.75
# c1, k1 = least_squares(residuals, (50, 10), bounds=((0.0, 0.0), (np.inf, np.inf))).x
fit.toc()

print('Ground matrix stiffness \'mu_g\': %.4f' % mu_g)
print('Elastin stiffness \'c1\': %.4f' % c1)
print('Collagen stiffness \'k1\': %.4f' % k1)

COD = fit.coefficient_of_determination(residuals((c1, k1)), sigma_exp)


# lmbd, sigma_exp = fit.data_preprocessing(csv_data, 1.0, np.inf, step=1)
# sigma_theo = uniaxial((c1, k1))
# data = [['Circumferential Stretch / 1', 'Experimental Circumferential Cauchy Stress / kPa', 'Theoretical Circumferential Cauchy Stress / kPa']] + \
#         [[lmbd[k], sigma_exp[k], sigma_theo[k]] for k in range(len(lmbd))]
# fit.write_csv(data, '/home/mwollner/Schreibtisch/Additional Material/Experimental Data/Uniaxial Passive/Media/data.csv')

# data = [['Ground Matrix / kPa', 'Collagen Fibre Stiffness / kPa', 'Elastic Fibre Stiffness / kPa', 'CoD / 1', 'Maximum Fitting Stretch / 1']] + \
#         [[mu_g, k1, c1, COD, lmbd_end]]
# fit.write_csv(data, '/home/mwollner/Schreibtisch/Additional Material/Experimental Data/Uniaxial Passive/Media/parameters.csv')

lmbd, sigma_exp = fit.data_preprocessing(csv_data, 1.0, np.inf, step=100)
ax1.plot(lmbd[::2], sigma_exp[::2], color='k', marker='+', linestyle='None', label='Experiment')
lmbd = np.linspace(1.0, 1.625, 500)
ax1.plot(lmbd, uniaxial((c1, k1)), color='k', label='Model', lw=1.0)

ax1.set_xlabel(r'Circumferential Stretch $\lambda_\varphi$ / 1')
ax1.set_xticks(np.arange(1.0, 1.65, 0.16))
ax1.set_xlim(1.0, 1.64)

ax1.set_ylabel(r'\textsc{Cauchy} Stress $\sigma^\mathrm{med}_{\varphi\varphi}$ / kPa')
ax1.set_ylim(0.0,1250.0)
ax1.set_yticks(np.arange(0.0, 1301.0, 250.0))

# ax1.legend(loc='upper left', fancybox=False, edgecolor='k')

fit.annotations(r'(a)', ax1, v=1.15)

# #==============================================================================
# plt.tight_layout(pad=0.0, w_pad=2.0)
# plt.savefig('/media/OS/Dokumente und Einstellungen/Maximilian Wollner/Documents/Studium/Diploma Thesis/Chapters/Regression of Mechanical Parameters/lsq_aorta.pdf')
# plt.show()







