import sys
sys.path.insert(0, '/media/OS/Users/Maximilian Wollner/Documents/Studium/Diploma Thesis/Python/Semi-analytical Medial Layer Model')

from scipy.optimize import least_squares

import matplotlib.pyplot as plt
import numpy as np
import fitting as fit

plt.close('all')
fit.latex_setup()
_, (ax1, ax2) = plt.subplots(1, 2, figsize=tuple(1*14*0.39370*np.array([1, 0.4])))

#==============================================================================

filename = '/media/OS/Dokumente und Einstellungen/Maximilian Wollner/Documents/Studium/Diploma Thesis/Python/Semi-analytical Medial Layer Model/Experimental Data/Collagen Fibre/Mechanical/Preprocessed Data/Collagen Fibre.csv'
csv_data = fit.read_csv(filename, 1)

lmbd = csv_data[:,0]
piola1st = 1e-3*csv_data[:,1]

filt = lmbd <= 1.36
lmbd = lmbd[filt]
eps = lmbd[lmbd <= 1.1] - 1
piola1st = piola1st[filt]

def uniaxial(parameter, linear=False):
    if linear == True:
        return 4*parameter*eps
    else:
        return 2*k1*(lmbd**2 - 1)*np.exp(parameter*(lmbd**2 - 1)**2)*lmbd

def residuals(parameters, linear=False):
    piola1st_theo = uniaxial(parameters, linear)
    return piola1st_theo - piola1st[:len(piola1st_theo)]

k1 = least_squares(residuals, max(piola1st), args=[True]).x[0]
k2 = least_squares(residuals, 3.).x[0]

print('Collagen fibre stiffness \'k1\': %.4f' % k1)
print('Collagen fibre shape parameter \'k2\': %.4f' % k2)
# print('Maximum relative error: %.4f' % (np.max(residuals(k2))))

# ax1.grid()

ax1.plot(lmbd[::5], piola1st[::5], color='k', marker='+', linestyle='None', label='Experiment')

fit.coefficient_of_determination(residuals(k2), piola1st)

lmbd = np.linspace(1.0, lmbd[-1], 500)
ax1.plot(lmbd, uniaxial(k2), color='k', label='Model', lw=1.0)

ax1.set_xlabel(r'Fibre Stretch $\lambda$ / 1')
ax1.set_xlim(1.0, 1.4)
ax1.set_xticks(np.arange(1.0, 1.41, 0.1))

ax1.set_ylabel(r'Nominal Stress $P^\mathrm{col}$ / MPa')
ax1.set_ylim(0.0, 200.0)
ax1.set_yticks(np.arange(0.0, 201.0, 50.0))

ax1.legend(loc='upper left', fancybox=False, edgecolor='k')

fit.annotations(r'(a)', ax1, v=1.15)
#==============================================================================
filename = '/media/OS/Dokumente und Einstellungen/Maximilian Wollner/Documents/Studium/Diploma Thesis/Python/Semi-analytical Medial Layer Model/Experimental Data/Elastic Fibre/Mechanical/Preprocessed Data/Elastic Fibre.csv'
csv_data = fit.read_csv(filename, 1)

lmbd = csv_data[:,0]
piola1st = csv_data[:,1]

filt = lmbd <= 2.08
lmbd = lmbd[filt]
piola1st = piola1st[filt]

def uniaxial(parameters):
    return parameters[0]*(lmbd**(parameters[1] - 1) - 1/lmbd)

def residuals(parameters):
    return uniaxial(parameters) - piola1st

c1, c2 = least_squares(residuals, (max(piola1st), 3.)).x

print('Elastic fibre stiffness \'c1\': %.4f' % c1)
print('Elastic fibre shape parameter \'c2\': %.4f' % c2)
# print('Maximum relative error: %.4f' % (np.max(residuals((c1,c2)) markersize=4,)))

# ax2.grid()

ax2.plot(lmbd[::3], piola1st[::3], color='k',marker='+', linestyle='None', label='Experiment')

fit.coefficient_of_determination(residuals((c1, c2)), piola1st)

lmbd = np.linspace(1.0, lmbd[-1], 500)
ax2.plot(lmbd, uniaxial((c1, c2)), color='k', label='Model', lw=1.0)

ax2.set_xlabel(r'Fibre Stretch $\lambda$ / 1')
ax2.set_xlim(1.0, 2.2)
ax2.set_xticks(np.arange(1.0, 2.21, 0.4))

ax2.set_ylabel(r'Nominal Stress $P^\mathrm{el}$ / kPa')
ax2.set_ylim(0.0, 500.0)
ax2.set_yticks(np.arange(0.0, 501.0, 100.0))

# ax2.legend(loc='upper left', fancybox=False, edgecolor='k')

fit.annotations(r'(b)', ax2, v=1.15)
#==============================================================================

plt.tight_layout(pad=0.1, w_pad=2.0)
plt.savefig('/media/OS/Dokumente und Einstellungen/Maximilian Wollner/Documents/Studium/Diploma Thesis/Chapters/Regression of Mechanical Parameters/lsq_collagen+elastic_fibre.pdf')
plt.show()




