import sys
sys.path.insert(0, '/media/OS/Users/Maximilian Wollner/Documents/Studium/Diploma Thesis/Python/Semi-analytical Medial Layer Model - Paper')
import matplotlib.pyplot as plt
import numpy as np
import fitting as fit
import preprocessing as pre

from scipy.optimize import least_squares, fsolve

n = 1
plt.close('all')
fit.latex_setup()
_, ax = plt.subplots(1, 1, figsize=tuple(1*14*0.39370*np.array([1.0, 0.5])))
ax.grid()
ax.set_xlabel(r'Circumferential Stretch $\lambda_\varphi$ / 1')
ax.set_ylabel(r'1st \textsc{Piola-Kirchhoff} Stress $\mathrm{P}_{\varphi\varphi}$ / kPa')
#================================== Passive ===================================
filename = "/media/OS/Dokumente und Einstellungen/Maximilian Wollner/Documents/Studium/Diploma Thesis/Python/Semi-analytical Medial Layer Model - Paper/Uniaxial Active/Scaled Experimental Data/Passive Smooth Muscle - Circumferential.csv"
csv_data = fit.read_csv(filename, 1)
lmbd = csv_data[::n,0]
piola1st_passive = csv_data[::n,1]
#================================== Active ====================================
filename = "/media/OS/Dokumente und Einstellungen/Maximilian Wollner/Documents/Studium/Diploma Thesis/Python/Semi-analytical Medial Layer Model - Paper/Uniaxial Active/Scaled Experimental Data/Active Smooth Muscle - Circumferential.csv"
csv_data = fit.read_csv(filename, 1)
piola1st_active = csv_data[::n,1]
#================================= Combined ===================================
piola1st = piola1st_passive + piola1st_active
directory = "/media/OS/Dokumente und Einstellungen/Maximilian Wollner/Documents/Studium/Diploma Thesis/Python/Semi-analytical Medial Layer Model - Paper/Uniaxial Active/"

def overlap(par):
    lmbd = np.linspace(0.5, 2.0, 500)
    I = lmbd**2
    mu_smc, I_eff, beta = par
    return mu_smc*(I/I_eff)**(beta - 1.0)*np.exp(-(beta - 1.0)/beta*((I/I_eff)**beta - 1.0))

fit.tic()
# theta0_list = tuple((theta0,) for theta0 in range(20, 60, 10))
theta0_list = ((0,),)
for theta0 in theta0_list:
    fit.tic()
    parameters0 = (150.0, 1.4, 2.5)
    media, _  = pre.constitutive_setup(theta0)
    media = media[0]
    def uniaxial_passive():
        lmbd_phi = lmbd    
        def traction_free_surfaces(x, y):
            lmbd_ax, = x
            F = fit.deformation_gradient(y, lmbd_ax, 0.)
            sigma_iso = media.isochoric_cauchy_stress(F)
            root = sigma_iso[0,0,0] - sigma_iso[0,2,2]
            return root            
        lmbd_ax = np.array([fsolve(traction_free_surfaces, 1., args=(y))[0] for y in lmbd_phi])      
        F = fit.deformation_gradient(lmbd_phi, lmbd_ax, np.zeros_like(lmbd_phi))    
        sigma_iso = media.isochoric_cauchy_stress(F)    
        piola1st_phiphi = (sigma_iso[:,1,1] - sigma_iso[:,2,2])/lmbd_phi
        return piola1st_phiphi, F
    
    piola1st_passive_theo, F0 = uniaxial_passive()
    
    # ax.plot(lmbd, piola1st_passive, color='k', marker='+', linestyle='none')
    # ax.plot(lmbd, piola1st_passive_theo, color='k', lw=1.0)
    
    # fit.coefficient_of_determination(piola1st_passive_theo-piola1st_passive, piola1st_passive)
    
    media.muscle_tone = 1.0
    
    def uniaxial_active(parameters):
        lmbd_phi = lmbd        
        # media.materials[-1].mu_smc, media.materials[-1].I_opt, media.materials[-1].alpha, media.materials[-1].beta = parameters        
        media.materials[-1].mu_smc, media.materials[-1].I_opt, media.materials[-1].beta = parameters        
        def traction_free_surfaces(x, y):
             lmbd_ax, gamma = x
             F = fit.deformation_gradient(y[1,1], lmbd_ax, gamma)
             sigma_iso = media.isochoric_cauchy_stress(F, [y])
             root = [sigma_iso[0,0,0] - sigma_iso[0,2,2], sigma_iso[0,0,1]]
             return root 
        lmbd_ax, gamma = np.array([fsolve(traction_free_surfaces, (1.0, -0.5), args=(y)) for y in F0]).T
        # print(lmbd_ax[-1], gamma[-1])
        F = fit.deformation_gradient(lmbd_phi, lmbd_ax, gamma)
        sigma_iso = media.isochoric_cauchy_stress(F, F0)
        # print(sigma_iso[0,0,0], F[0,:,:])
        piola1st_phiphi = (sigma_iso[:,1,1] - sigma_iso[:,2,2])/lmbd_phi
        return piola1st_phiphi, F
         
    def residuals(parameters):
        piola1st_phiphi, _ = uniaxial_active(parameters)
        return piola1st_phiphi - piola1st
    
    fit.tic()
    # results = least_squares(residuals, parameters0, bounds=((0.0, 0.0, 15.0, 0.8), (np.inf, np.inf, 25.0, 1.0))).x
    results = least_squares(residuals, parameters0, bounds=((0.0, 0.0, 0.0), (np.inf, np.inf, np.inf))).x
    # results = (321.8671867877693,1.1270362708025183,24.0,0.80)
    # results = (320.0, 1.1, 5.0)
    # mu_smc, I_opt, alpha, beta = results
    mu_smc, I_opt, beta = results
    
    fit.toc()
    
    print('Effective SMC stiffness \'mu_smc\': %.4f' % mu_smc)
    print('Effective SMC invariant \'I_opt\': %.4f' % I_opt)
    # print('First SMC shape parameter \'alpha\': %.4f' % alpha)
    print('Second SMC shape parameter \'beta\': %.4f' % beta)
    
    COD = fit.coefficient_of_determination(residuals(results), piola1st)
    ax.plot(lmbd, piola1st, color='k', marker='1', linestyle='none')

    lmbd = np.linspace(1.0,1.5,500)
    # lmbd = [1.50]
    piola1st_passive_theo, F0 = uniaxial_passive() 
    piola1st_theo, F_theo = uniaxial_active(results)
    # piola1st_theo, F_theo = uniaxial_active(results)
    
    ax.plot(lmbd, piola1st_theo, color='k', lw=1.0)
    
    plt.show()
    
    # fit.write_csv([('mu_smc / kPa', 'I_opt / 1', 'alpha / 1', 'beta / 1', 'CoD / 1'), [*results, COD]], directory+'Parameters - '+str(theta0[0])+'°.csv')
    fit.write_csv([('mu_smc / kPa', 'I_opt / 1', 'beta / 1', 'CoD / 1'), [*results, COD]], directory+'Parameters - '+str(theta0[0])+'°.csv')
    data = [["Smooth Muscle Cell Stretch / 1", "Contraction Efficiency / kPa"]] + [[lmbd_, over] for (lmbd_, over) in zip(np.linspace(0.5,2.0,500), overlap(results))]
    fit.write_csv(data, directory+'Overlap - '+str(theta0[0])+'°.csv')
    data = [['Circumferential Stretch / 1', 'Axial Stretch / 1', 'Shear Deformation / 1']] + [[F_theo[k,1,1], F_theo[k,2,2], F_theo[k,0,1]] for k in range(len(F_theo))]
    fit.write_csv(data, directory+'Uniaxial Deformation - '+str(theta0[0])+'°.csv')
    data = [['Circumferential Stretch / 1', 'Theoretical Passive 1st Piola-Kirchhoff Stress / kPa', 'Theoretical Total 1st Piola-Kirchhoff Stress / kPa']] + \
            [[lmbd[k], piola1st_passive_theo[k], piola1st_theo[k]] for k in range(len(lmbd))]
    fit.write_csv(data, directory+'Passive & Total Stress - '+str(theta0[0])+'°.csv')
    fit.toc()
# data = [['Circumferential Stretch / 1', 'Axial Stretch / 1', 'Shear Deformation / 1']] + [[F0[k,1,1], F0[k,2,2], F0[k,0,1]] for k in range(len(F0))]
# fit.write_csv(data, directory+'Uniaxial Deformation - passive.csv')
fit.toc()
       
    
    
    
