import sys
sys.path.insert(0, '/media/OS/Users/Maximilian Wollner/Documents/Studium/Diploma Thesis/Python/Semi-analytical Medial Layer Model')
from math import pi
from scipy.special import iv, erf, erfi
from scipy.optimize import fsolve

import numpy as np
import matplotlib.pyplot as plt
import fitting as fit


n = 1
plt.close('all')
fit.latex_setup()
_, (ax1, ax2) = plt.subplots(1, 2, figsize=tuple(1*14*0.39370*np.array([1, 0.5])))

#==============================================================================

filename = '/media/OS/Dokumente und Einstellungen/Maximilian Wollner/Documents/Studium/Diploma Thesis/Python/Semi-analytical Medial Layer Model/Experimental Data/Collagen Fibre/Histology/Preprocessed Data/In-plane Histogram - Adventitia.csv'
phiExp = pi/180*fit.read_csv(filename, 1)[:,0]
phi = np.linspace(-pi/2, pi/2, 500)

mean_cos = np.sum(np.cos(2*phiExp))/len(phiExp)
def cos_2phi0(a):
    return mean_cos*iv(0, a)/iv(1, a)
def sin_2phi0(a):
    return np.sqrt(1. - cos_2phi0(a)**2)
def root(a):
    try:
        a = a[0]
    except IndexError:
        None
    return -iv(1, a)/iv(0, a) + mean_cos*cos_2phi0(a) + sin_2phi0(a)*np.sum(np.sin(2*phiExp)*np.tanh(a*sin_2phi0(a)*np.sin(2*phiExp)))/len(phiExp)

a, = fsolve(root , 5.)
phi0 = 0.5*np.arccos(cos_2phi0(a))

print('Fibre mean angle \'phi0\': %.4f' % (180/pi*phi0))
print('Concentration parameter \'a\': %.4f' % a)
print('Number of occurences: %i' % len(phiExp))

pdf = 1/(2*iv(0, a))*( np.exp(a*np.cos(2*(phi - phi0))) + np.exp(a*np.cos(2*(phi + phi0))) )

phi *= 180/pi
phiExp *= 180/pi

height, bins = np.histogram(phiExp, np.linspace(-90., 90., 73), density=True)
locations = 0.5*(bins[1:]+bins[:-1])
height *= 180
width = n*(bins[1:]-bins[:-1])

ax1.bar(locations, height, width=width, color='tab:gray')
ax1.plot(phi, pdf, color='k', lw=1.)

# ax1.grid()

ax1.set_xlabel(r'In-plane Angle $\phi$ / ${}^\circ$')
ax1.set_xlim(-90.0, 90.0)
ax1.set_xticks(np.arange(-90.0, 90.1, 45.0))

ax1.set_ylabel(r'In-plane PDF $\rho_\mathrm{ip,col}$ / $1/{}^\circ$')
ax1.set_ylim(0.0,3.2)
ax1.set_yticks(np.arange(0.0, 3.6, 0.8))

fit.annotations(r'(a)', ax1)
#==============================================================================

filename = '/media/OS/Dokumente und Einstellungen/Maximilian Wollner/Documents/Studium/Diploma Thesis/Python/Semi-analytical Medial Layer Model/Experimental Data/Collagen Fibre/Histology/Preprocessed Data/Out-of-plane Histogram - Adventitia.csv'
thetaExp = pi/180*fit.read_csv(filename, 1)[:,0]
theta = np.linspace(-pi/2, pi/2, 500)

mean_cos = np.sum(np.cos(2*thetaExp))/len(thetaExp)
def root(parameters):
    b, theta0 = parameters

    _ = np.exp(b)*np.cos(theta0)*erf(np.sqrt(2*b)*np.cos(theta0)) + np.exp(-b)*np.sin(theta0)*erfi(np.sqrt(2*b)*np.sin(theta0))
    _dtheta0 = -np.exp( b)*np.sin(theta0)*( erf(np.sqrt(2*b)*np.cos(theta0)) + 2*np.sqrt(2*b/pi)*np.cos(theta0)*np.exp(-2*b*np.cos(theta0)**2)) \
               +np.exp(-b)*np.cos(theta0)*(erfi(np.sqrt(2*b)*np.sin(theta0)) + 2*np.sqrt(2*b/pi)*np.sin(theta0)*np.exp( 2*b*np.sin(theta0)**2))
    _db =  np.exp( b)*np.cos(theta0)*(  erf(np.sqrt(2*b)*np.cos(theta0)) + np.sqrt(2/(pi*b))*np.cos(theta0)*np.exp(-2*b*np.cos(theta0)**2)) \
          +np.exp(-b)*np.sin(theta0)*(-erfi(np.sqrt(2*b)*np.sin(theta0)) + np.sqrt(2/(pi*b))*np.sin(theta0)*np.exp( 2*b*np.sin(theta0)**2))

    dL_dtheta0 =  2*b*_*(np.sin(2*theta0)*mean_cos + np.cos(2*theta0)*np.sum(np.sin(2*thetaExp)*np.tanh(b*np.sin(2*theta0)*np.sin(2*thetaExp)))/len(thetaExp)) + _dtheta0
    dL_db = _*(1. + 2*b*(np.cos(2*theta0)*mean_cos + np.sin(2*theta0)*np.sum(np.sin(2*thetaExp)*np.tanh(b*np.sin(2*theta0)*np.sin(2*thetaExp)))/len(thetaExp))) - 2*b*_db

    return np.array([dL_db, dL_dtheta0])
 
b, theta0 = fsolve(root, np.array([35., 0.]))

print('Fibre mean angle \'theta0\': %.4f' % (180/pi*theta0))
print('Concentration parameter \'b\': %.4f' % b)
print('Number of occurences: %i' % len(thetaExp))

pdf = np.sqrt(2*b/pi)*(np.exp(b*np.cos(2*(theta - theta0))) + np.exp(b*np.cos(2*(theta + theta0))))/(np.exp(b)*np.cos(theta0)*erf(np.sqrt(2*b)*np.cos(theta0)) + np.exp(-b)*np.sin(theta0)*erfi(np.sqrt(2*b)*np.sin(theta0)))

theta *= 180/pi
thetaExp *= 180/pi

height, bins = np.histogram(thetaExp, np.linspace(-90., 90., 73), density=True)
locations = 0.5*(bins[1:]+bins[:-1])
height *= 360/pi/np.cos(pi/180*locations)
width = n*(bins[1:]-bins[:-1])

ax2.bar(locations, height, width=width, color='tab:gray')
ax2.plot(theta, pdf, color='k', lw=1.0)

# ax2.grid()

ax2.set_xlabel(r'Out-of-plane Angle $\theta$ / ${}^\circ$')
ax2.set_xlim(-90.0, 90.0)
ax2.set_xticks(np.arange(-90.0, 90.1, 45.0))

ax2.set_ylabel(r'Out-of-plane PDF $\rho_\mathrm{op,col}$ / $1/{}^\circ$')
ax2.set_ylim(0.0,14.0)
ax2.set_yticks(np.arange(0.0, 14.1, 3.5))

fit.annotations(r'(b)', ax2)
#==============================================================================

plt.tight_layout(pad=0.1, w_pad=2.0)
plt.savefig('/media/OS/Dokumente und Einstellungen/Maximilian Wollner/Documents/Studium/Diploma Thesis/Chapters/Regression of Histological Parameters/mle_adventitia.pdf')
plt.show()




