import sys
sys.path.insert(0, '/media/OS/Users/Maximilian Wollner/Documents/Studium/Diploma Thesis/Python/Semi-analytical Medial Layer Model')
from math import pi
from scipy.special import iv
from scipy.optimize import fsolve

import numpy as np
import matplotlib.pyplot as plt
from matplotlib.offsetbox import AnchoredText
import fitting as fit

n = 1
plt.close('all')
fit.latex_setup()
_, (ax2, ax1) = plt.subplots(1, 2, figsize=tuple(1*14*0.39370*np.array([1, 0.5])), gridspec_kw={'width_ratios': [1.5, 1]})

#==============================================================================

filename = '/media/OS/Dokumente und Einstellungen/Maximilian Wollner/Documents/Studium/Diploma Thesis/Python/Semi-analytical Medial Layer Model/Experimental Data/Smooth Muscle/Histology/Preprocessed Data/In-plane Histogram - SMC.csv'
phiExp = pi/180*fit.read_csv(filename, 1)[:,0]
phi = np.linspace(-pi/2, pi/2, 500)

mean_cos = np.sum(np.cos(2*phiExp))/len(phiExp)
def cos_2phi0(a):
    return mean_cos*iv(0, a)/iv(1, a)
phi0 = 0.
def root(a):
    return iv(1, a) - iv(0, a)*mean_cos

a, = fsolve(root , 4.)

print('Fibre mean angle \'phi0\': %.4f' % (180/pi*phi0))
print('Concentration parameter \'a\': %.4f' % a)
print('Number of occurences: %i' % len(phiExp))

pdf = 1/(2*iv(0, a))*( np.exp(a*np.cos(2*(phi - phi0))) + np.exp(a*np.cos(2*(phi + phi0))) )

phi *= 180/pi
phiExp *= 180/pi

height, bins = np.histogram(phiExp, np.linspace(-90., 90., 37), density=True)
locations = 0.5*(bins[1:]+bins[:-1])
height *= 180
width = n*(bins[1:]-bins[:-1])


ax1.bar(locations, height, width=width, color='tab:gray')
ax1.plot(phi, pdf, color='k', lw=1.)

# ax1.grid()

ax1.set_xlabel(r'In-plane Angle $\phi$ / ${}^\circ$')
ax1.set_xlim(-90.0, 90.0)
ax1.set_xticks(np.arange(-90.0, 90.1, 45.0))

ax1.set_ylabel(r'In-plane PDF $\rho_\mathrm{ip,smc}$ / $1/{}^\circ$')
ax1.set_ylim(0.0,7)
ax1.set_yticks(np.arange(0.0, 8, 3.5/2))

at = AnchoredText(r'(b)', frameon=False, bbox_to_anchor=(-0.15*3/2.5,1.1), loc='center', bbox_transform=ax1.transAxes)
ax1.add_artist(at)
# fit.annotations(r'(b)', ax1)

#==============================================================================

phi = np.linspace(-pi, pi, 500)
uniform = pi/180*np.ones(len(phi))
def pdf(phi0, a):
    return pi/180*1/(4*iv(0,a))*np.exp(a*np.cos(2*(phi - phi0)))

pdfs = np.array([pdf(x, 2) for x in [0, pi/2, pi/4, -pi/4]])
pdf_combined = np.sum(pdfs, axis=0)

labels = []
# ax2.grid()

linestyles = ((0, (5, 1, 1, 1)), (0, (5, 1, 1, 1, 1, 1)), (0, (5, 1, 1, 1, 1, 1, 1, 1)), (0, (5, 1, 1, 1, 1, 1, 1, 1, 1, 1)))
phi *= 180/pi
labels = (r'$0^\circ$', r'$90^\circ$', r'$45^\circ$', r'$-45^\circ$')
for k in (0,2,1,3):
    if k == 0:
        ax2.plot(phi, uniform, color="k", label="Uniform", linestyle=(0, (5, 1)), lw=1.)
    if k == 1:
        ax2.plot(phi, pdf_combined, color="k", label="Superposition", lw=1.)
    ax2.plot(phi, pdfs[k], color="k", linestyle=linestyles[k], label=labels[k], lw=1.)


ax2.set_xlabel(r'In-plane Angle $\phi$ / ${}^\circ$')
ax2.set_xlim(-90.0, 90.0)
ax2.set_xticks(np.arange(-90.0, 90.1, 45.0))

ax2.set_ylabel(r'In-plane PDF $\rho_\mathrm{ip,el}$ / $1/{}^\circ$')
ax2.set_yticks(np.arange(0.0, 0.031, 0.01))
ax2.set_ylim(bottom=0.0)
ax2.legend(loc='upper center', ncol=2, fancybox=False, edgecolor='k')

at = AnchoredText(r'(a)', frameon=False, bbox_to_anchor=(-0.15*2/2.5,1.1), loc='center', bbox_transform=ax2.transAxes)
ax2.add_artist(at)
# fit.annotations(r'(a)', ax2)

plt.tight_layout(pad=0.1, w_pad=2.0)
plt.savefig('/media/OS/Dokumente und Einstellungen/Maximilian Wollner/Documents/Studium/Diploma Thesis/Chapters/Regression of Histological Parameters/mle_smc+elastic_fibres.pdf')
plt.show()




