import sys
sys.path.insert(0, '/media/OS/Users/Maximilian Wollner/Documents/Studium/Diploma Thesis/Python/Semi-analytical Medial Layer Model')
import numpy as np
import matplotlib.pyplot as plt
import fitting as fit

plt.close('all')
fit.latex_setup()

def angle(lmbd_phi, lmbd_z, gamma, theta0):
    from math import pi
    theta0 *= pi/180
    m = np.array((np.sin(theta0)/(lmbd_phi*lmbd_z), (gamma*np.sin(theta0) + lmbd_phi*np.cos(theta0)), 0))
    m_ = 1/np.sqrt(m@m)*m
    return 180/pi*np.arctan(m[0]/m[1])

linestyles = ((0, (5, 1, )), (0, (5, 1, 1, 1)), ':', (0, (5, 1, 1, 1, 1, 1, 1, 1)))
_, (ax1, ax2) = plt.subplots(1, 2, figsize=tuple(1*14*0.39370*np.array([1, 0.5])))
# ax1.grid()
ax1.set_xlabel(r'Circumferential Stretch $\lambda_\varphi$ / 1')
ax1.set_ylabel(r'Nominal Stress $P_{\varphi\Phi}$ / kPa')

# ax2.grid()
ax2.set_xlabel(r'SMC Stretch $\lambda_{0,\mathrm{smc}}$ / 1')
ax2.set_ylabel(r'Contraction Efficiency $\mu_\mathrm{smc}\mathcal{O}$ / kPa')
# linestyles = ((0, 5, 1, 1, 1)), (0, (5, 1, 1, 1, 1, 1)), (0, (5, 1, 1, 1, 1, 1, 1, 1)), (0, (5, 1, 1, 1, 1, 1, 1, 1, 1, 1)))


def overlap(par):
    lmbd = np.linspace(0.5, 2.0, 1000)
    I = lmbd**2
    mu_smc, I_eff, alpha, beta = par
    return mu_smc*(I/I_eff)**alpha*np.exp(-alpha/beta*((I/I_eff)**beta - 1.0))

directory = '/media/OS/Dokumente und Einstellungen/Maximilian Wollner/Documents/Studium/Diploma Thesis/Python/Semi-analytical Medial Layer Model/Parameters/'
lmbd, passive_exp, passive_theo, total_exp, total_theo = fit.read_csv(directory+'Passive & Total Stress - 20°.csv', 1).T
ax1.plot(lmbd, passive_exp, color='k', linestyle='none', marker='.', markersize=4, label='Passive')
ax1.plot(lmbd, passive_theo, lw=1.0, linestyle='-', color='k')
ax1.plot(lmbd, total_exp, marker='+', color='k', linestyle='none', label='Active')
ax1.plot(lmbd, total_theo, lw=1.0, linestyle='-', color='k', label='Model')

ax1.legend(loc='upper left', fancybox=False, edgecolor='k')

ax1.set_xticks(np.linspace(1.0, 1.4, 5))
ax1.set_xlim(1.0, 1.4)

ax1.set_yticks(np.linspace(0.0, 300, 4))
ax1.set_ylim(0.0, 300)

fit.annotations(r'(a)', ax1)

for k, theta0 in enumerate((0, 30, 20, 40)):    
    par = fit.read_csv(directory+'Active Parameters - '+str(theta0)+'°.csv', 1).T
    ax2.plot(np.linspace(0.5, 1.8, 500), overlap(par), lw=1.0, linestyle=linestyles[k], color='k', label=str(theta0)+r'${}^\circ$')
    
ax2.legend(loc='upper left', fancybox=False, edgecolor='k', ncol=2)

ax2.set_xticks(np.linspace(0.6, 1.6, 6))
ax2.set_xlim(0.6, 1.6)

ax2.set_yticks(np.linspace(0.0, 300, 4))
ax2.set_ylim(0.0, 300)

fit.annotations(r'(b)', ax2)  

plt.tight_layout(pad=0.1, w_pad=2.0)  
plt.savefig('/media/OS/Dokumente und Einstellungen/Maximilian Wollner/Documents/Studium/Diploma Thesis/Chapters/Regression of Mechanical Parameters/active_fit.pdf')  
plt.show()

_, (ax4, ax3) = plt.subplots(1, 2, figsize=tuple(1*14*0.39370*np.array([1, 0.5])))
# ax3.grid()
ax3.set_xlabel(r'Circumferential Stretch $\lambda_\varphi$ / 1')
ax3.set_ylabel(r'Amount of Shear $\gamma$ / 1')

# ax4.grid()
ax4.set_xlabel(r'Circumferential Stretch $\lambda_\varphi$ / 1')
ax4.set_ylabel(r'Axial Stretch $\lambda_z$ / 1')

_, lmbd_ax, gamma = fit.read_csv(directory+'Uniaxial Deformation - passive.csv', 1).T

ax3.plot(lmbd, gamma, lw=1.0, linestyle='-', color='k', label=r'Passive / $0^\circ$')
labels = (r'$30^\circ$', r'$20^\circ$', r'$40^\circ$')
linestyles = ((0, (5, 1, 1, 1)), ':', (0, (5, 1, 1, 1, 1, 1, 1, 1)))
for k, theta0 in enumerate((30, 20, 40)):      
    _, lmbd_ax, gamma = fit.read_csv(directory+'Uniaxial Deformation - '+str(theta0)+'°.csv', 1).T
    

_, lmbd_ax, gamma = fit.read_csv(directory+'Uniaxial Deformation - passive.csv', 1).T

ax4.plot(lmbd, lmbd_ax, lw=1.0, linestyle='-', color='k', label=r'Passive')
labels = (r'$0^\circ$', r'$30^\circ$', r'$20^\circ$', r'$40^\circ$')
linestyles = (':', (0, (5, 1, 1, 1, 1, 1, 1, 1)), (0, (5, 1, )), (0, (5, 1, 1, 1)))
for k, theta0 in enumerate((20, 40, 0, 30)):      
    _, lmbd_ax, gamma = fit.read_csv(directory+'Uniaxial Deformation - '+str(theta0)+'°.csv', 1).T
    ax4.plot(lmbd, lmbd_ax, lw=1.0, linestyle=linestyles[k], color='k', label=str(theta0)+r'${}^\circ$')
    ax3.plot(lmbd, gamma, lw=1.0, linestyle=linestyles[k], color='k', label=str(theta0)+r'${}^\circ$')
    print(angle(lmbd[0], lmbd_ax[0], gamma[0], theta0))
    
ax3.set_xticks(np.linspace(1.0, 1.4, 5))
ax3.set_xlim(1.0, 1.4)

ax3.set_yticks(np.linspace(-0.7, 0.1, 5))
ax3.set_ylim(-0.7, 0.1)

# ax3.legend(loc='upper left', fancybox=False, edgecolor='k', ncol=2)

fit.annotations(r'(b)', ax3)

ax4.set_xticks(np.linspace(1.0, 1.4, 5))
ax4.set_xlim(1.0, 1.4)

ax4.set_yticks(np.linspace(0.5, 1.3, 5))
ax4.set_ylim(0.5, 1.3)

ax4.legend(loc='upper left', fancybox=False, edgecolor='k', ncol=2)

fit.annotations(r'(a)', ax4)

plt.tight_layout(pad=0.1, w_pad=2.0)  
plt.savefig('/media/OS/Dokumente und Einstellungen/Maximilian Wollner/Documents/Studium/Diploma Thesis/Chapters/Regression of Mechanical Parameters/active_deformation.pdf')  
plt.show()
    
