import sys
sys.path.insert(0, '/media/OS/Users/Maximilian Wollner/Documents/Studium/Diploma Thesis/Python/Semi-analytical Medial Layer Model - Paper')
import matplotlib.pyplot as plt
import numpy as np
import fitting as fit
import preprocessing as pre

from scipy.optimize import least_squares, fsolve

plt.close('all')
fit.latex_setup()
_, (ax2, ax1) = plt.subplots(1, 2, figsize=tuple(1*14*0.39370*np.array([1.0, 0.5])))
#================================== Passive ===================================
filename = '/media/OS/Dokumente und Einstellungen/Maximilian Wollner/Documents/Studium/Diploma Thesis/Python/Semi-analytical Medial Layer Model - Paper/Experimental Data/Smooth Muscle/Mechanical/Porcine/Preprocessed Data/Passive Smooth Muscle - Circumferential.csv'
csv_data = fit.read_csv(filename, 1)
lmbd_passive = csv_data[:,0]
piola1st_passive = csv_data[:,1]
#================================== Active ====================================
filename = '/media/OS/Dokumente und Einstellungen/Maximilian Wollner/Documents/Studium/Diploma Thesis/Python/Semi-analytical Medial Layer Model - Paper/Experimental Data/Smooth Muscle/Mechanical/Porcine/Preprocessed Data/Active Smooth Muscle - Circumferential.csv'
csv_data = fit.read_csv(filename, 1)
lmbd_active = csv_data[:,0]
piola1st_active = csv_data[:,1]
#================================= Combined ===================================
piola1st_porcine = piola1st_passive + piola1st_active
lmbd_porcine = np.mean([lmbd_passive, lmbd_active], axis=0)

media, _  = pre.constitutive_setup(tuple([0]))
media = media[0]
def uniaxial(henky_offset):
    lmbd_human = lmbd_porcine**henky_offset

    def traction_free_surfaces(x, y):
        lmbd_trans, = x
        F = fit.deformation_gradient(y, lmbd_trans, 0.0)
        sigma_iso = media.isochoric_cauchy_stress(F)
        root = sigma_iso[0,0,0] - sigma_iso[0,2,2]
        return root
    
    lmbd_trans = np.array([fsolve(traction_free_surfaces, 1., args=(y))[0] for y in lmbd_human])  
    F = fit.deformation_gradient(lmbd_human, lmbd_trans, np.zeros_like(lmbd_human))    
    sigma_iso = media.isochoric_cauchy_stress(F)    
    sigma_human = sigma_iso[:,1,1] - sigma_iso[:,2,2]
    
    piola1st_human = sigma_human/lmbd_human
    return piola1st_human
    
def residuals(offset):
    offset_stress, offset_henky = offset
    return 1/offset_stress*uniaxial(offset_henky)[1:] - piola1st_passive[1:]

fit.tic()
# offset_stress = 2.3145
# offset_henky = 0.5069
offset_stress, offset_henky = least_squares(residuals, (2.3, 0.5)).x
fit.toc()

print('Stress offset: %.4f' % offset_stress)
print('Stretch offset: %.4f' % offset_henky)

COD = fit.coefficient_of_determination(residuals((offset_stress, offset_henky)), piola1st_passive)

lmbd_human = lmbd_porcine**offset_henky
piola1st_human = offset_stress*piola1st_porcine

ax1.plot(lmbd_porcine[::2], piola1st_passive[::2], color='k', linestyle='none', marker='.', markersize=4, label='Passive')
ax1.plot(lmbd_porcine[::2], piola1st_porcine[::2], color='k', linestyle='none', marker='+', label='Total')

ax1.set_xticks(np.linspace(1.0, 2.0, 6))
ax1.set_xlim(1.0, 2.0)

ax1.set_yticks(np.linspace(0.0, 150, 4))
ax1.set_ylim(0.0, 150.0)

ax1.set_xlabel(r'Porcine Stretch $\lambda_{\varphi}^\mathrm{por}$ / 1')
ax1.set_ylabel(r'Porcine Nominal Stress $P^\mathrm{por}_{\varphi\Phi}$ / kPa')

# ax1.legend(loc='upper left', fancybox=False, edgecolor='k')
fit.annotations(r'(b)', ax1)

piola1st_passive *= offset_stress
piola1st_active *= offset_stress

ax2.plot(lmbd_human[::2], piola1st_passive[::2], color='k', linestyle='none', marker='.', markersize=4, label='Passive')
ax2.plot(lmbd_human[::2], piola1st_human[::2], color='k', linestyle='none', marker='+', label='Total')
ax2.plot(lmbd_human, uniaxial(offset_henky), color='k', lw=1.0, label='Model')

ax2.set_xticks(np.linspace(1.0, 1.4, 5))
ax2.set_xlim(1.0, 1.4)

ax2.set_xlabel(r'Human Stretch $\lambda_{\varphi}^\mathrm{hum}$ / 1')
ax2.set_ylabel(r'Human Nominal Stress $P^\mathrm{hum}_{\varphi\Phi}$ / kPa')

ax2.legend(loc='upper left', fancybox=False, edgecolor='k')
fit.annotations(r'(a)', ax2)

ax2.set_yticks(np.linspace(0.0, 300, 4))
ax2.set_ylim(0.0, 300)

plt.tight_layout(pad=0.1, w_pad=2.0)
# plt.savefig('/media/OS/Dokumente und Einstellungen/Maximilian Wollner/Documents/Studium/Diploma Thesis/Chapters/Regression of Mechanical Parameters/smc_scaling.pdf')
plt.show()

scaling = [["Stress Offset / 1", "Henky Strain 0ffset / 1", "CoD / 1"], [offset_stress, offset_henky, COD]]
fit.write_csv(scaling, "/media/OS/Dokumente und Einstellungen/Maximilian Wollner/Documents/Studium/Diploma Thesis/Python/Semi-analytical Medial Layer Model - Paper/Uniaxial Active/Scaled Experimental Data/Scaling Offsets.csv")
passive = [['Circumferential Stretch / 1', 'Circumferential 1st Piola-Kirchhoff Stress / kPa']] + [[lmbd_human[k], piola1st_passive[k]] for k in range(len(lmbd_human))]
fit.write_csv(passive, "/media/OS/Dokumente und Einstellungen/Maximilian Wollner/Documents/Studium/Diploma Thesis/Python/Semi-analytical Medial Layer Model - Paper/Uniaxial Active/Scaled Experimental Data/Passive Smooth Muscle - Circumferential.csv")
active = [['Circumferential Stretch / 1', 'Circumferential 1st Piola-Kirchhoff Stress / kPa']] + [[lmbd_human[k], piola1st_active[k]] for k in range(len(lmbd_human))]
fit.write_csv(active, "/media/OS/Dokumente und Einstellungen/Maximilian Wollner/Documents/Studium/Diploma Thesis/Python/Semi-analytical Medial Layer Model - Paper/Uniaxial Active/Scaled Experimental Data/Active Smooth Muscle - Circumferential.csv")
# piola1st_human_theo = uniaxial(offset_henky)
# theo = [['Circumferential Stretch / 1', 'Circumferential 1st Piola-Kirchhoff Stress / kPa']] + [[lmbd_human[k], piola1st_human_theo[k]] for k in range(len(lmbd_human))]
# fit.write_csv(theo, "/media/OS/Dokumente und Einstellungen/Maximilian Wollner/Documents/Studium/Diploma Thesis/Python/Semi-analytical Medial Layer Model - Paper/Uniaxial Active/Passive Fit.csv")
