from csv import reader, writer
from time import time
from math import floor

import matplotlib.pyplot as plt
from matplotlib.offsetbox import AnchoredText
import numpy as np
import preprocessing as pre

def tic():
    global start
    try:
        start.append(time())
    except NameError:
        start = [time()]

def toc():
    duration = time() - start[-1]
    start.pop()
    if duration > 60.:
        minutes = floor(duration/60)
        seconds = duration - 60*minutes
        print('Elapsed time: %i minutes and %f seconds.\n' % (minutes, seconds))
    else:
        print('Elapsed time: %f seconds.\n' % duration)

def read_csv(filename, skip_header=0, delimiter=','):
    data = []
    with open(filename, 'r') as file:
        csv = reader(file, delimiter=delimiter)
        for row in range(skip_header):
            next(csv)
        for row in csv:
            data.append(np.array([float(entry) for entry in row]))
        data = np.array(data)
    return data

def write_csv(data, filename):
    with open(filename, 'w', newline='') as file:
        csv = writer(file, delimiter=',')
        csv.writerows(data)

def latex_setup():
    plt.rcParams.update({
        "axes.axisbelow": True,
        "font.family": "serif",
        "font.size": 9,
        "font.serif": [],
        "font.sans-serif": [],
        "text.usetex": True,
        "text.latex.preamble": [r"\usepackage{upgreek}", r"\usepackage{amsmath}"]
    })
    
def ppp_setup():
    plt.rcParams.update({
        "axes.axisbelow": True,
        "font.family": "sans-serif",
        "font.size": 14,
        "font.serif": [],
        "font.sans-serif": ['Open Sans'],
        "text.usetex": True,
        "text.latex.preamble": [r"\usepackage{amsmath}"]
    })
    
def annotations(text, ax, h=-0.15, v=1.1):
    at = AnchoredText(text, frameon=False, bbox_to_anchor=(h, v), loc='center', bbox_transform=ax.transAxes)
    ax.add_artist(at)

def data_preprocessing(data, lmbd_min, lmbd_max, step=0, skip=0):
    lmbd_exp = data[skip::step,2]/data[0,2]
    sigma_exp = data[skip::step,1] -  data[0,1]
    filt = (lmbd_exp >= lmbd_min)*(lmbd_exp <= lmbd_max)
    lmbd_exp = lmbd_exp[filt]
    sigma_exp = sigma_exp[filt]
    return lmbd_exp, sigma_exp

def coefficient_of_determination(residuals, experiment):
    SS_res = np.sum(residuals**2)
    SS_tot = np.sum((experiment - np.mean(experiment))**2)
    COD = 1 - SS_res/SS_tot
    print('Coefficient of determination \'R2\': %.4f' % COD)
    return COD

def deformation_gradient(lmbd_1, lmbd_2, gamma):
        try:
            F = np.array([ np.array([1/(lmbd_1[k]*lmbd_2[k]), gamma[k], 0., 0., lmbd_1[k], 0., 0., 0., lmbd_2[k]]).reshape((3, 3)) for k in range(len(lmbd_1)) ])
        except TypeError:
            F = np.array([ np.array([1/(lmbd_1   *lmbd_2   ), gamma   , 0., 0., lmbd_1   , 0., 0., 0., lmbd_2   ]).reshape((3, 3))])
        return F


