import solver as sol
import preprocessing as pre
import matplotlib.pyplot as plt

from fitting import tic, toc

plt.close('all')
#++++++++++++++++++++++++++++++++ Radial Tilt +++++++++++++++++++++++++++++++++
theta0 = (20.0, -20.0)

#==============================================================================
#                            Residual Stresses                                
#==============================================================================

#+++++++++++++++++++++++++++++ Preprocessing ++++++++++++++++++++++++++++++++++
BC = {'natural': (0.0, 0.0, 0.0),
      'essential': (None, 228.8506)
      }
residual = pre.residual_setup(BC, theta0)

#+++++++++++++++++++++++++++++++++ Solver +++++++++++++++++++++++++++++++++++++
tic()
residual.BVP_solver()
toc()

#+++++++++++++++++++++++++++++ Postprocessing +++++++++++++++++++++++++++++++++
residual.plot_results()

#==============================================================================
#                                In-vivo                                      
#==============================================================================
lmbd_z = pre.axial_prestretch(50)

#+++++++++++++++++++++++++++++ Preprocessing ++++++++++++++++++++++++++++++++++
BC = {'natural': (90.0, 0.0, None),
      'essential': (lmbd_z, 0.0)
      }
invivo = pre.invivo_setup('In-vivo', BC, residual)

#+++++++++++++++++++++++++++++++++ Solver +++++++++++++++++++++++++++++++++++++
tic()
invivo.BVP_solver()
toc()

#+++++++++++++++++++++++++++++ Postprocessing +++++++++++++++++++++++++++++++++
invivo.plot_results()

#==============================================================================
#                                In-vivo                                      
#==============================================================================
lmbd_z = pre.axial_prestretch(50)

#+++++++++++++++++++++++++++++ Preprocessing ++++++++++++++++++++++++++++++++++
BC = {'natural': (90.0, 0.0, None),
      'essential': (lmbd_z, 0.0)
      }
invivo = pre.invivo_setup('In-vivo', BC, theta0)
# invivo = pre.invivo_setup('In-vivo', BC, residual)

#+++++++++++++++++++++++++++++++++ Solver +++++++++++++++++++++++++++++++++++++
tic()
invivo.BVP_solver()
toc()

#+++++++++++++++++++++++++++++ Postprocessing +++++++++++++++++++++++++++++++++
invivo.plot_results()



