import constitutive as con
import bodies as bod
import solver as sol
import numpy as np

from copy import deepcopy
from csv import reader

def axial_prestretch(age):
    assert (age > 0.), 'The patients age must be greater than zero.'
    lmbd_z = 2.3969*age**(-0.1952)
    if lmbd_z < 1.:
        lmbd_z = 1.
    return lmbd_z

def read_csv(filename, skip_header=0, delimiter=','):
    data = []
    with open(filename, 'r') as file:
        csv = reader(file, delimiter=delimiter)
        for row in range(skip_header):
            next(csv)
        for row in csv:
            data.append(np.array([float(entry) for entry in row]))
        data = np.array(data)
    return data


def constitutive_setup(theta0):
    #=============================== Adventitia ===============================
    materials_adventitia = (con.ground_matrix('gr', 6.184758424788485),
                            con.collagen('col1', (27.86033255634872, 1.3807), "/media/OS/Users/Maximilian Wollner/Documents/Studium/Diploma Thesis/Python/Semi-analytical Medial Layer Model - Paper/Parameters/DFD/o_cf_adventitia_a=3,08,b=33,20,-50,72.csv"),
                            con.collagen('col2', (27.86033255634872, 1.3807), "/media/OS/Users/Maximilian Wollner/Documents/Studium/Diploma Thesis/Python/Semi-analytical Medial Layer Model - Paper/Parameters/DFD/o_cf_adventitia_a=3,08,b=33,20,+50,72.csv")                           
                            )
    adventitia = tuple([bod.layer('adventitia', materials_adventitia)])
    #================================== Media =================================
    materials_media = [con.ground_matrix('gr', 6.184758424788485),
                       con.collagen('col1', (10.747695031484769, 1.3807), "/media/OS/Users/Maximilian Wollner/Documents/Studium/Diploma Thesis/Python/Semi-analytical Medial Layer Model - Paper/Parameters/DFD/o_cf_media_a=3,62,b=34,30,-25,14.csv"),
                       con.collagen('col2', (10.747695031484769, 1.3807), "/media/OS/Users/Maximilian Wollner/Documents/Studium/Diploma Thesis/Python/Semi-analytical Medial Layer Model - Paper/Parameters/DFD/o_cf_media_a=3,62,b=34,30,+25,14.csv"),
                       con.elastin('el1', (62.966863740906795, 2.9841), "/media/OS/Users/Maximilian Wollner/Documents/Studium/Diploma Thesis/Python/Semi-analytical Medial Layer Model - Paper/Parameters/DFD/o_ef_b=34,30.csv"),  
                       ]
    media = []
    for k, theta0_LU in enumerate(theta0):
        par_hist_smc = "/media/OS/Users/Maximilian Wollner/Documents/Studium/Diploma Thesis/Python/Semi-analytical Medial Layer Model - Paper/Parameters/DFD/o_smc_a=4,07,b=100,"+str(abs(theta0_LU))+".csv"
        par_mech_smc = 3*(None,)
        materials_LU = materials_media + [con.smooth_muscle('smc'+str(k+1), par_mech_smc, par_hist_smc)]
        media += [bod.layer('LU'+str(k+1), materials_LU)]
    media = tuple(media)
    #==========================================================================
    return media, adventitia
    
def geometry_setup(theta0):
    geometry = tuple(np.linspace(19.08/2, 21.64/2, len(theta0)+1)) + (22.7/2,)
    # geometry = (10.0, 15.0, 20.0)
    return geometry

def residual_setup(BC, theta0): 
    #================================ Geometry ================================
    geometry = {'reference': None,
                'current': geometry_setup(theta0)
                }
    #============================== Constitutive ==============================
    media, adventitia = constitutive_setup(theta0)
    layers = media + adventitia
    
    residual = sol.model('Residual', layers, geometry, BC)
    return residual

def invivo_setup(name, BC, auxiliary): 
    #================================ Geometry ================================
    try:
        geometry = {'reference': deepcopy(auxiliary.geometry['current']),
                    'current': None
                     }
    except AttributeError:
        geometry = {'reference': geometry_setup(auxiliary),
                   'current': None
                    }
    #============================== Constitutive ==============================
    try:
        layers = []
        for k, layer in enumerate(auxiliary.layers):
            layers.append(bod.layer(layer.name, layer.materials))
            layers[-1].save_residual_stresses(layer.r, layer.sigma)
        layers = tuple(layers)
    except AttributeError:
        media, adventitia = constitutive_setup(auxiliary)
        layers = media + adventitia
        
    invivo = sol.model(name, layers, geometry, BC)
    
    return invivo

def active_setup(name, BC, invivo, muscle_tone): 
    #================================ Geometry ================================
    geometry = {'reference': deepcopy(invivo.geometry['reference']),
                'current': None
                 }
    #============================== Constitutive ==============================
    layers = []
    for k, layer in enumerate(invivo.layers):
        layers.append(bod.layer(layer.name, layer.materials))
        layers[-1].save_in_vivo_deformation(layer.reference_radius(layer.r), layer.lmbd_phi, layer.lmbd_z)
        if k < (len(invivo.layers)-1):
            layers[-1].muscle_tone = muscle_tone
    layers = tuple(layers)
          
    active = sol.model(name, layers, geometry, BC)
    
    return active
    






